// Generated by CoffeeScript 1.7.1
(function() {
  var $, Annotator, Store, _t;

  Annotator = require('annotator');

  $ = Annotator.Util.$;

  _t = Annotator.Util.TranslationString;

  Store = (function() {
    Store.prototype.options = {
      annotationData: {},
      emulateHTTP: false,
      emulateJSON: false,
      headers: {},
      prefix: '/store',
      urls: {
        create: '/annotations',
        read: '/annotations/:id',
        update: '/annotations/:id',
        destroy: '/annotations/:id',
        search: '/search'
      }
    };

    function Store(options) {
      this.options = $.extend(true, {}, this.options, options);
    }

    Store.prototype.create = function(annotation) {
      return this._apiRequest('create', annotation);
    };

    Store.prototype.update = function(annotation) {
      return this._apiRequest('update', annotation);
    };

    Store.prototype["delete"] = function(annotation) {
      return this._apiRequest('destroy', annotation);
    };

    Store.prototype.query = function(queryObj) {
      var dfd;
      dfd = $.Deferred();
      this._apiRequest('search', queryObj).done(function(obj) {
        var rows;
        rows = obj.rows;
        delete obj.rows;
        return dfd.resolve(rows, obj);
      }).fail(function() {
        return dfd.reject.apply(dfd, arguments);
      });
      return dfd.promise();
    };

    Store.prototype.setHeader = function(key, value) {
      return this.options.headers[key] = value;
    };

    Store.prototype._apiRequest = function(action, obj) {
      var id, options, request, url;
      id = obj && obj.id;
      url = this._urlFor(action, id);
      options = this._apiRequestOptions(action, obj);
      request = $.ajax(url, options);
      request._id = id;
      request._action = action;
      return request;
    };

    Store.prototype._apiRequestOptions = function(action, obj) {
      var data, method, opts;
      method = this._methodFor(action);
      opts = {
        type: method,
        dataType: "json",
        error: this._onError,
        headers: this.options.headers
      };
      if (this.options.emulateHTTP && (method === 'PUT' || method === 'DELETE')) {
        opts.headers = $.extend(opts.headers, {
          'X-HTTP-Method-Override': method
        });
        opts.type = 'POST';
      }
      if (action === "search") {
        opts = $.extend(opts, {
          data: obj
        });
        return opts;
      }
      data = obj && JSON.stringify(obj);
      if (this.options.emulateJSON) {
        opts.data = {
          json: data
        };
        if (this.options.emulateHTTP) {
          opts.data._method = method;
        }
        return opts;
      }
      opts = $.extend(opts, {
        data: data,
        contentType: "application/json; charset=utf-8"
      });
      return opts;
    };

    Store.prototype._urlFor = function(action, id) {
      var url;
      url = this.options.prefix != null ? this.options.prefix : '';
      url += this.options.urls[action];
      url = url.replace(/\/:id/, id != null ? '/' + id : '');
      url = url.replace(/:id/, id != null ? id : '');
      return url;
    };

    Store.prototype._methodFor = function(action) {
      var table;
      table = {
        create: 'POST',
        read: 'GET',
        update: 'PUT',
        destroy: 'DELETE',
        search: 'GET'
      };
      return table[action];
    };

    Store.prototype._onError = function(xhr) {
      var action, message;
      action = xhr._action;
      message = _t("Sorry we could not ") + action + _t(" this annotation");
      if (xhr._action === 'search') {
        message = _t("Sorry we could not search the store for annotations");
      } else if (xhr._action === 'read' && !xhr._id) {
        message = _t("Sorry we could not ") + action + _t(" the annotations from the store");
      }
      switch (xhr.status) {
        case 401:
          message = _t("Sorry you are not allowed to ") + action + _t(" this annotation");
          break;
        case 404:
          message = _t("Sorry we could not connect to the annotations store");
          break;
        case 500:
          message = _t("Sorry something went wrong with the annotation store");
      }
      Annotator.showNotification(message, Annotator.Notification.ERROR);
      return console.error(_t("API request failed:") + (" '" + xhr.status + "'"));
    };

    return Store;

  })();

  Annotator.Plugin.register('Store', Store);

  module.exports = Store;

}).call(this);
